''***************************************
''* WindNeedle_Demo_v1.0.spin    4/2018 *
''* Author: Werner L. Schneider         *
''***************************************
''
''
CON
    _clkmode      = xtal1 + pll16x
    _xinfreq      = 5_000_000

    video      = 16                      ' P16-P23 VGA

    '' Keyboard data and clock
    kbd        = 26
    kbc        = 27

  
VAR
 
    byte fontbuff[2048]                ' Font-Buffer 256 Char's 8x8 Pixel

    byte vgabuff[2400]                 ' VGA Text-Buffer 40x30 1200x2 Bytes 1. Byte = Character, 2. Byte = Color (bgc * 16 +fgc)
    byte cursor[6]                     ' Cursor info array
    long sync                          ' sync used by VGA routine
 
    long pos                           ' Global Screen-Pointer

    word ms                            ' Timer
    byte speed                         ' Current wind speed


OBJ

    kb:     "Keyboard"                 ' Keyboard driver        need 1 Cog

    vga:    "VGACOLOR_320x240_CURSOR"  ' VGA Driver             need 2 Cog's

'    shot:   "SCREENSHOT"
    

PUB main | k, dir     

    init

    initScreen

    ms := 0
    
    dir := 0                                  ' wind direction 0 - 15
    updateDir(dir)

    repeat

        k := kb.key
        if k <> 0
            if k == $C2                       ' Arrow Up

                if dir < 15
                    dir++
                else
                    dir := 0
                updateDir(dir)
                
            elseif k == $C3                   ' Arrow Down

                if dir > 0
                    dir--
                else
                    dir := 15
                updateDir(dir)

'            elseif k == $D0                   ' F1
'                shot.Screenshot320(@vgabuff, @fontbuff)

        waitcnt((clkfreq / 1000) * 1 + cnt)   ' wait 1 ms

        ms++
        if ms == 1000                         ' Update every second
            ms := 0

            updateSpeed

            updateMaxMinute
        
            updateMaxRecord

            updateTime

            updateTemp

            updateBarom
            

PRI updateSpeed

    speed := 4 + ((||(cnt?)) // 7)                  ' Random 4 - 10                     orange on black
    decxy(28, 6, $08, $00, speed)

    
PRI updateMaxMinute

    decxy(29, 8, $0D, $00, 16)                      '                                   magenta on black


PRI updateMaxRecord

    decxy(30, 10, $09, $00, 34)                     '                                   green on black


PRI updateTime

    str(31, 12, $0B, $00, string("09:25"))          ' Print "09:25"                     yellow on black


PRI updateTemp

    str(6, 15, $0A, $00, string("68.20"))           ' Print "68.00"                     green on black
    str(11, 15, $0A, $00, @strF)                    ' Print "F"                        green on black
                

PRI updateBarom

    str(23, 15, $08, $00, string("29,87"))          ' Print "29,87"                     orange on black


PRI updateDir(d) | Winput

    Winput := wintable[d]
    binxy(20, 18, $0E, $00, Winput, 8)              ' Print binary Winput               cyan on black
    str(18, 13, $04, $00, @dirtable + (d * 5))      ' Print N, N NE...                  white on black

    case Winput
        %1000_0000:
            updateNeedle(@wind_N)
        %1000_0001:
            updateNeedle(@wind_NNE)
        %0000_0001:
            updateNeedle(@wind_NE)
        %0000_0011:
            updateNeedle(@wind_ENE)
        %0000_0010:
            updateNeedle(@wind_E)
        %0000_0110:
            updateNeedle(@wind_ESE)
        %0000_0100:
            updateNeedle(@wind_SE)
        %0000_1100:
            updateNeedle(@wind_SSE)
        %0000_1000:
            updateNeedle(@wind_S)
        %0001_1000:
            updateNeedle(@wind_SSW)
        %0001_0000:
            updateNeedle(@wind_SW)
        %0011_0000:
            updateNeedle(@wind_WSW)
        %0010_0000:
            updateNeedle(@wind_W)
        %0110_0000:
            updateNeedle(@wind_WNW)
        %0100_0000:
            updateNeedle(@wind_NW)
        %1100_0000:
            updateNeedle(@wind_NNW)



PRI updateNeedle(ptr) | x, y

    repeat y from 19 to 25
        repeat x from 2 to 8
            printxy(x, y, $0F, $00, byte[ptr++])            ' Print Needle             white on black


PRI init | i, j, k, p, offs
''************************************************
''* Start Keyboard, VGA Cog's, Clear Screen and  *
''* set Cursor's                                 *
''************************************************

    kb.start(kbd, kbc)                                      ' Start Keyboard Driver  

    offs := 62                                              ' 62 Bytes BMP-Header not used, 128x128 2 Color Bitmap
    repeat i from 15 to 0
        repeat j from 7 to 0
            p := (i * 16) + (j * 256)
            repeat k from 0 to 15
                fontbuff[p+k] := reverse(font[offs++])


    wordfill(@vgabuff, $0F20, 1200)                         ' Clear Screen with $20 (Space) FG-Color $F (white), BG-Color $0 (black)

    vga.start(video, @vgabuff, @fontbuff, @cursor, @sync)   ' Start Video Driver
    waitcnt(clkfreq * 1 + cnt)                              ' wait 1 second for cogs to start

    str(0, 0, $0B, $00, @strVersion1)                       ' Print "Wind-Needle Demo"     yellow on black
    str(0, 2, $0B, $00, @strVersion2)                       ' Print "v1.0 ws 4/2018"       yellow on black

    printHat($0A)                                           ' Print Propeller Hat          green on black

    ifnot (kb.present)                                      ' Keyboard present?
        str(0, 4, $09, $00, @strNoKeyboard)                 ' Print "No Keyboard "         red on black
        cursor[0] := 11                             
        cursor[1] := 4
        cursor[2] := %010
        repeat

    cursor[0] := 0                             
    cursor[1] := 0
    cursor[2] := 0  '%010                                   ' Cursor 1 = Off

    cursor[3] := 1
    cursor[4] := 0
    cursor[5] := 0  '%111                                   ' Cursor 2 = Off


PRI printHat(color) | i

    repeat i from 0 to 6
        printxy(33+i, 0, color, $00, $C0+i)                 '                              green on black
        printxy(33+i, 1, color, $00, $D0+i)                 '                              green on black
        printxy(33+i, 2, color, $00, $E0+i)                 '                              green on black
        printxy(33+i, 3, color, $00, $F0+i)                 '                              green on black
        printxy(33+i, 4, color, $00, $C7+i)                 '                              green on black


PRI initScreen

    str(0, 6, $0F, $00, @strWindSpeed)         ' Print "Current wind speed in MPH ="       white on black
    str(0, 8, $0F, $00, @strWindMax)           ' Print "Max wind in current minute ="      white on black
    str(0, 10, $0F, $00, @strWindRecorded)     ' Print "Maximum wind recorded today ="     white on black
    str(28, 12, $0F, $00, @strWindAt)          ' Print "at"                                white on black
    str(1, 13, $0F, $00, @strWindFrom)         ' Print "Wind is from the "                 white on black
    str(1, 15, $0F, $00, @strTemp)             ' Print "Temp"                              white on black
    str(16, 15, $0F, $00, @strBarom)           ' Print "Barom."                            white on black


    drawBox(0, 17, 11, 11, $04, $00)

    printxy(5, 18, $0F, $00, "N")              ' Print "N"                                 white on black
    printxy(5, 26, $0F, $00, "S")              ' Print "S"                                 white on black
    printxy(1, 22, $0F, $00, "W")              ' Print "W"                                 white on black
    printxy(9, 22, $0F, $00, "E")              ' Print "E"                                 white on black

    str(12, 18, $0F, $00, @strWinput)          ' Print "Winput"                            white on black

    str(12, 25, $07, $00, @strPress1)          ' Print "Press + or - to"                   grey on black
    str(12, 26, $07, $00, @strPress2)          ' Print "to change Wind Direction"          grey on black

    str(0, 29, $0B, $00, @strStar)             ' Print "****************************************"   yellow on black


PRI reverse(val) : new | i
''************************************************
''* Reverse a Byte                               *
''* eg. %01000000 = $40 to %00000010 = $02       * 
''************************************************

    i := 1
    val <<= 32 - 8
    repeat 8
        if ((val <-= 1) & 1) == 1
            new += i 
        i *= 2 


PRI getPos(x, y)

    return (x + y * 40) * 2


PRI print(fgc, bgc, char) 
''************************************************
''* Print Character char at current pos          *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    vgabuff[pos++] := char
    vgabuff[pos++] := bgc * 16 + fgc


PRI printxy(x, y, fgc, bgc, char) 
''************************************************
''* Print Character char at current pos          *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    pos := getPos(x, y)
    print(fgc, bgc, char)


PRI str(x, y, fgc, bgc, string_ptr)
''************************************************
''* Print String at x, y                         *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    pos := (x + y * 40) * 2
    repeat strsize(string_ptr)
        print(fgc, bgc, byte[string_ptr++])



PRI drawBox(x, y, w, h, fgc, bgc) | t, m, b, i
''************************************************
''* Draw Box at x, y with w = width, h = height  *
''* Foreground.Color fgc                         *
''* Background-Color bgc                         *
''************************************************

    t := (y * 40 + x) * 2
    m := t + 80
    b := m + ((h-2) * 80)
    pos := t  
    print(fgc, bgc, $80)               
    fill(fgc, bgc, $81, w-2)           
    print(fgc, bgc, $82)               
    repeat i from 0 to h-3
        pos := m + i * 80
        print(fgc, bgc, $86)
        fill(fgc, bgc, $20, w-2)
        print(fgc, bgc, $86)   
    pos := b   
    print(fgc, bgc, $83)                
    fill(fgc, bgc, $81, w-2)           
    print(fgc, bgc, $84)


PRI fill(fgc, bgc, char, len) | x
''************************************************
''* Fill len of Char's with char                 *
''* Foreground.Color fgc                         *
''* Background-Color bgc                         *
''* at current pos                               * 
''************************************************

    if len <> 0
        repeat x from 1 to len
            print(fgc, bgc, char)


PRI decxy(x, y, fg, bg, val) 

    pos := getPos(x, y)
    prn(fg, bg, val)
    print(fg, bg, $20)


PRI prn(fg, bg, val) | dig

    dig := 48 + (val // 10)
    val := val/10
    if val > 0
        prn(fg, bg, val)
    print(fg, bg, dig)


PRI binxy(x, y, fgc, bgc, value, digits)
''************************************************
''* Print binary value at x, y                   *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    pos := (x + y * 40) * 2
    value <<= 32 - digits
    repeat digits
        print(fgc, bgc, (value <-= 1) & 1 + "0")


DAT

strVersion1       byte "Wind-Needle Demo", 0
strVersion2       byte "v1.0 ws 4/2018", 0
strNoKeyboard     byte "No Keyboard ", 0
strWindSpeed      byte "Current wind speed in MPH =", 0
strWindMax        byte "Max wind in current minute =", 0
strWindRecorded   byte "Maximum wind recorded today =", 0
strWindAt         byte "at", 0
strWindFrom       byte "Wind is from the ", 0
strTemp           byte "Temp", 0  
strF              byte $BF, "F", 0
strBarom          byte "Barom.", 0  
strWinput         byte "Winput: ", 0
strPress1         byte "Press ", $BD, " or ", $BE, " to",0
strPress2         byte "change Wind Direction", 0
strStar           byte "****************************************", 0


wintable          byte %1000_0000
                  byte %1000_0001
                  byte %0000_0001
                  byte %0000_0011
                  byte %0000_0010
                  byte %0000_0110
                  byte %0000_0100
                  byte %0000_1100
                  byte %0000_1000
                  byte %0001_1000
                  byte %0001_0000
                  byte %0011_0000
                  byte %0010_0000
                  byte %0110_0000
                  byte %0100_0000
                  byte %1100_0000


dirtable          byte "N   ", 0
                  byte "N NE", 0
                  byte "NE  ", 0
                  byte "E NE", 0
                  byte "E   ", 0
                  byte "E SE", 0
                  byte "SE  ", 0
                  byte "S SE", 0
                  byte "S   ", 0
                  byte "S SW", 0
                  byte "SW  ", 0
                  byte "W SW", 0
                  byte "W   ", 0
                  byte "W NW", 0
                  byte "NW  ", 0
                  byte "N NW", 0


wind_N            byte $20, $20, $20, $88, $20, $20, $20
                  byte $20, $20, $20, $86, $20, $20, $20
                  byte $20, $20, $20, $86, $20, $20, $20
                  byte $20, $20, $20, $85, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_NNE          byte $20, $20, $20, $20, $8F, $20, $20
                  byte $20, $20, $20, $20, $8E, $20, $20
                  byte $20, $20, $20, $8C, $8D, $20, $20
                  byte $20, $20, $20, $85, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_NE           byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $92, $93, $20
                  byte $20, $20, $20, $92, $93, $91, $20
                  byte $20, $20, $20, $90, $91, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_ENE          byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $96, $97, $98
                  byte $20, $20, $20, $94, $95, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_E            byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $85, $87, $87, $8B
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_ESE          byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $85, $99, $20, $20
                  byte $20, $20, $20, $20, $9A, $9B, $9C
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_SE           byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $9D, $9E, $20, $20
                  byte $20, $20, $20, $9F, $A0, $9E, $20
                  byte $20, $20, $20, $20, $9F, $A0, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_SSE          byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $A1, $20, $20, $20
                  byte $20, $20, $20, $A2, $A3, $20, $20
                  byte $20, $20, $20, $20, $A4, $20, $20
                  byte $20, $20, $20, $20, $A5, $20, $20

wind_S            byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $85, $20, $20, $20
                  byte $20, $20, $20, $86, $20, $20, $20
                  byte $20, $20, $20, $86, $20, $20, $20
                  byte $20, $20, $20, $89, $20, $20, $20

wind_SSW          byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $85, $20, $20, $20
                  byte $20, $20, $A7, $A6, $20, $20, $20
                  byte $20, $20, $A8, $20, $20, $20, $20
                  byte $20, $20, $A9, $20, $20, $20, $20

wind_SW           byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $92, $AA, $20, $20, $20
                  byte $20, $92, $93, $91, $20, $20, $20
                  byte $20, $93, $91, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_WSW          byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $AC, $AB, $20, $20, $20
                  byte $AF, $AE, $AD, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_W            byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $8A, $87, $87, $85, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_WNW          byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $B3, $B2, $B1, $20, $20, $20, $20
                  byte $20, $20, $B0, $85, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_NW           byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $A0, $9E, $20, $20, $20, $20
                  byte $20, $9F, $A0, $9E, $20, $20, $20
                  byte $20, $20, $9F, $B4, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20

wind_NNW          byte $20, $20, $B9, $20, $20, $20, $20
                  byte $20, $20, $B8, $20, $20, $20, $20
                  byte $20, $20, $B7, $B6, $20, $20, $20
                  byte $20, $20, $20, $B5, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20
                  byte $20, $20, $20, $20, $20, $20, $20




font              file  "CP437_8.BMP"


{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                   TERMS OF USE: Parallax Object Exchange License                                             |
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    |
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}
                       '